#!/bin/bash
# Script to fetch package sources

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi
require_script git_utils.sh

targetDir=$DOWNLOAD_LOC
srcUrl=$SRC_LOC
srcSha256=$SRC_SHA256
srcMd5=$SRC_MD5
srcFilename=${srcUrl##*/}
srcFile="$targetDir/$srcFilename"

do_buildstep_prepend

gen_sha256() {
	local src="$1"
	local computed_sha256=$(sha256sum "$srcFile")
	echo ${computed_sha256%% *}
}

gen_md5() {
	local src="$1"
	local computed_md5=$(m5sum "$srcFile")
	echo ${computed_md5%% *}
}
	
check_downloaded_file() {
	local srcFile="$1"
	local cksumerror=${2:-false}
	case $srcUrl in
		git://* | git+ssh://* | ssh://* | http://*.git | https://*.git)
			# $srcFile.done is updated by the git function
			update_git_repository "$srcUrl" "$targetDir" "$srcFilename" "$SRC_REV" "$SRC_BRANCH"
			;;
		http://* | https://* | ftp://*)
			if [ ! -r "$srcFile" ]; then
				return 1
			fi
			computed_sha256=$(gen_sha256 "$srcFile")
			computed_md5=$(gen_md5 "$srcFile")
			if [ -n "$srcSha256" ]; then
				if [ "$srcSha256" != "$computed_sha256" ]; then
					rm -f $srcFile.done
					if cksumerror; then
						fatal 1 "Sha256 checksum for '$srcFile' is wrong: expected '$srcSha256', found '$computed_sha256'"
					else
						warning "Found '$srcFile' but sha256 checksum is wrong: expected '$srcSha256', found '$computed_sha256'"
						return 1
					fi
				fi
			else
				warning "No sha256 checksum provided for '$srcUrl'; think at adding one for security"
				warning "Sha256 checksum for '$srcUrl' should be '$computed_sha256'"
			fi
			if [ -n "$srcMd5" ]; then
				if [ "$srcMd5" != "$computed_md5" ]; then
					rm -f $srcFile.done
					if cksumerror; then
						fatal 1 "SMd5 checksum for '$srcFile' is wrong: expected '$srcMd5', found '$computed_md5'"
					else
						warning "Found '$srcFile' but md5 checksum is wrong: expected '$srcMd5', found '$computed_md5'"
						return 1
					fi
				fi
			else
				warning "No md5 checksum provided for '$srcUrl'; think at adding one for security"
				warning "Md5 checksum for '$srcUrl' should be '$computed_md5'"
			fi
			(echo "MD5=$computed_md5"; echo "SHA256=$computed_sha256") > $srcFile.done
			;;
		*:*)
			rm -f $srcFile.done
			return 1
			;;
		*)
			if [ -r "$srcFile" ]; then
				gen_sha256 "$srcFile" > $srcFile.done
			elif [ -r "$srcFile" ]; then
				gen_sha256 "$srcFile" > $srcFile.done
			elif [ -r "$srcFile-$PACKAGE_VERSION.tar".* ]; then
				gen_sha256 "$srcFile-$PACKAGE_VERSION.tar".* > $srcFile-$PACKAGE_VERSION.done
			elif [ -r "$srcFile.tar".* ]; then
				gen_sha256 "$srcFile.tar".* > $srcFile.done
			else
				rm -f $srcFile.done
				return 1
			fi
			;;
	esac
}

if check_downloaded_file "$srcFile"; then
	log "'$srcFilename' has been found in '$targetDir'"
else
	case $srcUrl in
		git://* | git+ssh://* | ssh://* | http://*.git | https://*.git)
			# $srcFile.done is updated by the git function
			clone_git_repository "$srcUrl" "$targetDir" "$srcFilename" "$SRC_REV" "$SRC_DATE" "$SRC_BRANCH"
			;;
		http://* | https://* | ftp://*)
			log "Downloading $srcUrl ... into $targetDir"
			rm -f "$srcFile" # Just in case the file exist but is not readable
			wget -nv -nc -P "$targetDir" "$srcUrl"
			check_downloaded_file "$srcFile" true
			;;
		*:*)
			rm -f $srcFile.done
			fatal 1 "Unsupported scheme: '${srcUrl%%:*}:'"
			;;
		*)
			if [ "$srcUrl" = "empty" ]; then
				srcDir="/tmp/fetch.$$"
				srcFilename="$srcUrl"
				srcUrl="$srcDir/$srcFilename"
				mkdir -p "$srcUrl"
			fi
			if [ -d "$srcUrl" ]; then
				if [ "$PACKAGE_VERSION" != "none" ]; then
					srcFile=${srcFile%-$PACKAGE_VERSION}-$PACKAGE_VERSION
				fi
				if [ -r "$srcFile.tar.bz2" ]; then
					echo "'${srcFile##*/}.tar.bz2' has been found in '$targetDir'"
				else
					srcDir=${srcUrl%/*}
					echo "Packing '$srcUrl' in '$srcFile.tar.bz2'"
					tar cjf "$srcFile.tar.bz2" -C "$srcDir" "$srcFilename"
				fi
				gen_sha256 "$srcFile.tar.bz2" > $srcFile.done
			else
				echo "Copying '$srcUrl' in '$targetDir'"
				cp "$srcUrl" "$targetDir"
				gen_sha256 "$srcFile" > $srcFile.done
			fi
			;;
	esac
fi

do_buildstep_append
